from transformers import AutoTokenizer

from incognito import Incognito, PrivateTokenizer


def test_incognito():
    model = Incognito()

    input = "My name is John Doe, I live in New York and I work at Starbucks."
    label = f"My name is {model.NAME}, I live in {model.LOCATION} and I work at {model.ORGANIZATION}."
    output = model(input)

    assert output == label

    batch_input = [input, input]
    batch_label = [label, label]
    batch_output = model(batch_input)

    assert batch_output == batch_label


def test_private_tokenizer():
    tokenizer = AutoTokenizer.from_pretrained("hf-internal-testing/tiny-random-bert")
    private_tokenizer = PrivateTokenizer.from_pretrained("hf-internal-testing/tiny-random-bert")

    input = "My name is John Doe, I live in New York and I work at Starbucks."
    label = f"My name is [NAME], I live in [LOCATION] and I work at [ORGANIZATION]."

    input_tokens = tokenizer.decode(tokenizer(input).input_ids)
    label_tokens = tokenizer.decode(tokenizer(label).input_ids)
    private_input_tokens = private_tokenizer.decode(private_tokenizer(input).input_ids)

    assert label_tokens == private_input_tokens
    assert input_tokens != label_tokens
    assert input_tokens != private_input_tokens

    batch_input = [input, input]
    batch_label = [label, label]

    batch_input_tokens = tokenizer.batch_decode(tokenizer(batch_input).input_ids)
    batch_label_tokens = tokenizer.batch_decode(tokenizer(batch_label).input_ids)
    batch_private_input_tokens = private_tokenizer.batch_decode(private_tokenizer(batch_input).input_ids)

    assert batch_label_tokens == batch_private_input_tokens
    assert batch_input_tokens != batch_label_tokens
    assert batch_input_tokens != batch_private_input_tokens



from typing import Union, List

from transformers import TokenClassificationPipeline, AutoModelForTokenClassification, AutoTokenizer


class PrivateTokenizer(AutoTokenizer):

    @classmethod
    def from_pretrained(cls, pretrained_model_name_or_path, *model_args, **kwargs):
        auto_tokenizer = super().from_pretrained(pretrained_model_name_or_path, *model_args, **kwargs)
        cls.incognito = Incognito()

        tokenizer_encode_plus = auto_tokenizer.encode_plus
        tokenizer_batch_encode_plus = auto_tokenizer.batch_encode_plus

        # Patch the encode_plus method
        def patched_encode_plus(text, *args, **kwargs):
            return tokenizer_encode_plus(
                text=cls.incognito(text), *args, **kwargs
            )

        auto_tokenizer.encode_plus = patched_encode_plus

        def patched_batch_encode_plus(batch_text_or_text_pairs, *args, **kwargs):
            return tokenizer_batch_encode_plus(
                batch_text_or_text_pairs=cls.incognito(batch_text_or_text_pairs), *args, **kwargs
            )

        auto_tokenizer.batch_encode_plus = patched_batch_encode_plus

        return auto_tokenizer

    @classmethod
    def incognito(cls, inputs):
        raise NotImplementedError("Cannot use incognito method before calling from_pretrained.")


class Incognito:
    def __init__(self, name="[NAME]", location="[LOCATION]", organization="[ORGANIZATION]"):
        self.NAME = name
        self.LOCATION = location
        self.ORGANIZATION = organization

        self.ner = TokenClassificationPipeline(
            model=AutoModelForTokenClassification.from_pretrained("Davlan/distilbert-base-multilingual-cased-ner-hrl"),
            tokenizer=AutoTokenizer.from_pretrained("Davlan/distilbert-base-multilingual-cased-ner-hrl"),
            aggregation_strategy="simple",
        )

    def __call__(self, input: Union[str, List[str]]):

        converted = False
        if isinstance(input, str):
            converted = True
            input = [input]

        input_entities = self.ner(input)

        for i, entities in enumerate(input_entities):
            entities = sorted(entities, key=lambda x: x['start'], reverse=True)

            for entity in entities:
                if entity['entity_group'] == 'ORG':
                    placeholder = self.ORGANIZATION
                elif entity['entity_group'] == 'PER':
                    placeholder = self.NAME
                elif entity['entity_group'] == 'LOC':
                    placeholder = self.LOCATION
                else:
                    continue
                input[i] = input[i][:entity['start']] + placeholder + input[i][entity['end']:]

        if converted:
            return input[0]
        return input



#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages

REQUIREMENTS = ["transformers", "torch"]

TEST_REQUIREMENTS = ["pytest"]
QUALITY_REQUIREMENTS = ["black", "ruff"]

EXTRAS_REQUIREMENTS = {
    "dev": TEST_REQUIREMENTS + QUALITY_REQUIREMENTS,
    "test": TEST_REQUIREMENTS,
}
setup(
    author="FairNLP",
    author_email="info@fairnlp.com",
    python_requires=">=3.7",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    description="Out-of-the-box anonymization for NLP.",
    install_requires=REQUIREMENTS,
    license="Apache Software License 2.0",
    include_package_data=True,
    keywords="incognito",
    name="incognito",
    packages=find_packages(include=["incognito", "incognito.*"]),
    test_suite="tests",
    tests_require=TEST_REQUIREMENTS,
    extras_require=EXTRAS_REQUIREMENTS,
    url="https://github.com/FairNLP/incognito",
    version="0.0.1",
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    zip_safe=False,
)
